@@ -37,15 +37,14 @@ diagnostic_from_lint <- function(result) {
 }
 
 # copy lintr:::find_config to here since CRAN doesn't like :::
-find_config <- function (filename) {
+find_config <- function(filename) {
     if (is.null(filename)) {
         return(NULL)
     }
     linter_file <- getOption("lintr.linter_file")
     path <- if (is_directory(filename)) {
         filename
-    }
-    else {
+    } else {
         dirname(filename)
     }
     linter_config <- file.path(path, linter_file)
@@ -57,11 +56,11 @@ find_config <- function (filename) {
     if (isTRUE(file.exists(linter_config))) {
         return(linter_config)
     }
-    home_dir <- Sys.getenv("HOME", unset = "~")
-    linter_config <- file.path(home_dir, linter_file)
-    if (isTRUE(file.exists(linter_config))) {
-        return(linter_config)
-    }
+    # home_dir <- Sys.getenv("HOME", unset = "~")
+    # linter_config <- file.path(home_dir, linter_file)
+    # if (isTRUE(file.exists(linter_config))) {
+    #     return(linter_config)
+    # }
     NULL
 }
 
